# Devices

In Wirehome.Core a "device" is any kind of physical device, machine or thing which is part of the Wirehome.Core controlled installation. Each device can report it's status to the core and may also support several standardized commands (like starting a firmware update etc.).

## Properties

The following status keys are predefined in Wirehome. Custom ones may be added. when needed. Each value has some meta information like the timestamp when it was set.

| UID | Type | Example |
|-|-|-|
| firmware_version | string | "1.0.0" |
| network_address | string | "192.168.1.2" |
| supports_firmware_update | boolean | true |
| model | string | "HSRel5" |
| manufacturer | string | "CCTools" |
| first_seen* | datetime (ISO) | "2018.18.11 33:44:55" |
| last_update* | datetime (ISO) | "2018.18.11 33:44:55" |
| wifi_reconnects | number | 25 |
| wifi_rssi | number | -80 |
| mqtt_reconnects | number | 50 |
| mqtt_client_id | number | 50 |
| bus_address | string | 32 |
| info_url | string | "https://cctools.eu/hsrel5" |

\* These are generated by Wirehome.Core automatically.

## Custom action

A device can report a set of custom actions. These are actions which can be triggered in the UI. It is possible to register HTTP and MQTT based actions. The device has to provide a URI or MQTT message definition which will be published when the action is being invoked in the UI.

## Format

```json
{
    "reported": {
        "firmware_version": {
            "timestamp": "2013-04-04 12:30:30.123",
            "value": "123"
        },
        "update_interval": {
            "timestamp": "...",
            "value": 3
        }
    },
    "requested": {
        "update_interval": {
            "timestamp": "2019-01-01",
            "value": 3
        }
    }
}
```